/*
 *     CFG.java  Apr 2, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.cfg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;
import org.w3c.dom.*;

public class CFG extends Graph {
    private CFGMethodEntry start;
    private CFGExit end;

    public CFG(Node node) {
        GetCFG cfgCreator = new GetCFG(this);
        cfgCreator.create(node);
    }

    public void setStartNode(CFGMethodEntry node) {
        start = node;
    }

    public CFGMethodEntry getStartNode() {
        return start;
    }

    public void setEndNode(CFGExit node) {
        end = node;
    }

    public CFGExit getEndNode() {
        return end;
    }

    public void add(CFGNode node) {
        super.add(node);
    }

    public void add(Flow edge) {
        super.add(edge);
    }

    private GraphCompSet track = new GraphCompSet();  // CFGNode

    public GraphCompSet getForwardReachableNodes(CFGNode fromNode, CFGNode toNode) {
        track.clear();
        walkForward(fromNode, toNode, true);
        return track;
    }

    public GraphCompSet getForwardReachableNodesWithoutLoopback(CFGNode fromNode, CFGNode toNode) {
        track.clear();
        walkForward(fromNode, toNode, false);
        return track;
    }

    public GraphCompSet getBackwardReachableNodes(CFGNode fromNode, CFGNode toNode) {
        track.clear();
        walkBackward(fromNode, toNode, true);
        return track;
    }

    public GraphCompSet getBackwardReachableNodesWithoutLoopback(CFGNode fromNode, CFGNode toNode) {
        track.clear();
        walkBackward(fromNode, toNode, false);
        return track;
    }

    private void walkForward(GraphNode fromNode, GraphNode toNode, boolean loopOk) {
        if (fromNode.equals(toNode) && !track.isEmpty()) {
            track.add(fromNode);
            return;
        }
        track.add(fromNode);

        Iterator it = fromNode.getOutgoingEdges().iterator();
        while (it.hasNext()) {
            Flow flow = (Flow)it.next();
            if (loopOk || !flow.isLoopBack()) {

                GraphNode succ = flow.getDstNode();
                if (!track.contains(succ)) {
                    walkForward(succ, toNode, loopOk);
                }
            }
        }
    }

    private void walkBackward(GraphNode toNode, GraphNode fromNode, boolean loopOk) {
        if (toNode.equals(fromNode) && !track.isEmpty()) {
            track.add(toNode);
            return;
        }
        track.add(toNode);

        Iterator it = toNode.getIncomingEdges().iterator();
        while (it.hasNext()) {
            Flow flow = (Flow)it.next();
            if (loopOk || !flow.isLoopBack()) {

                GraphNode pred = flow.getSrcNode();
                if (!track.contains(pred)) {
                    walkBackward(pred, fromNode, loopOk);
                }
            }
        }
    }

    public Flow getFlow(CFGNode src, CFGNode dst) {
        if (src != null && dst != null) {
            Iterator it = edges.iterator();
            while (it.hasNext()) {
                Flow edge = (Flow)it.next();
                if (src.equals(edge.getSrcNode()) && dst.equals(edge.getDstNode())) {
                    return edge;
                }
            }
        }
        return null;
    }

    public GraphCompSet getFlowsTo(CFGNode dst) {
        GraphCompSet set = new GraphCompSet();
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getDstNode().equals(dst)) {
                set.add(edge);
            }
        }
        return set;
    }

    public GraphCompSet getFlowsFrom(CFGNode src) {
        GraphCompSet set = new GraphCompSet();
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(src)) {
                set.add(edge);
            }
        }
        return set;
    }

    public GraphCompSet getSrcNodes(CFGNode dst) {
        GraphCompSet set = new GraphCompSet();
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getDstNode().equals(dst)) {
                set.add(edge.getSrcNode());
            }
        }
        return set;
    }

    public GraphCompSet getDstNodes(CFGNode src) {
        GraphCompSet set = new GraphCompSet();
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(src)) {
                set.add(edge.getDstNode());
            }
        }
        return set;
    }

    public CFGNode getNode(Element elem) {
        Iterator it = nodes.iterator();
        while(it.hasNext()) {
            CFGNode node = (CFGNode)it.next();
            if (elem.equals(node.getElem())) {
                return node;
            }
        }
        return null;
    }

    public CFGNode getNode(int id) {
        Iterator it = nodes.iterator();
        while(it.hasNext()) {
            CFGNode node = (CFGNode)it.next();
            if (id == node.getId()) {
                return node;
            }
        }
        return null;
    }

    public Flow getTrueFlow(CFGNode node) {
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(node) && edge.isTrue()) {
                return edge;
            }
        }
        return null;
    }

    public Flow getFalseFlow(CFGNode node) {
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(node) && edge.isFalse()) {
                return edge;
            }
        }
        return null;
    }

    public CFGNode getTrueSuccessor(CFGNode node) {
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(node) && edge.isTrue()) {
                return (CFGNode)edge.getDstNode();
            }
        }
        return null;
    }

    public CFGNode getFalseSuccessor(CFGNode node) {
        Iterator it = edges.iterator();
        while (it.hasNext()) {
            Flow edge = (Flow)it.next();
            if (edge.getSrcNode().equals(node) && edge.isFalse()) {
                return (CFGNode)edge.getDstNode();
            }
        }
        return null;
    }

    public void print() {
        StringBuffer buf = new StringBuffer();
        buf.append("----- CFG (from here) -----\n");
        printNodes(buf);
        printEdges(buf);
        buf.append("----- CFG (to here) -----\n");

        System.out.print(buf);
    }

    public String write() {
        StringBuffer buf = new StringBuffer();
        buf.append("<CFG");
        JavaMethod jm = new JavaMethod(start.getElem());
        buf.append(" class=\"" + jm.getJavaClass().getFqn() + "\"");
        buf.append(" method=\"" + jm.getSignature() + "\"");
        buf.append(">\n");
        writeNodes(buf);
        writeEdges(buf);
        buf.append("</CFG>\n");
        return buf.toString();
    }

    public void writeNodes(StringBuffer buf) {
        buf.append("  <nodes>\n");

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            GraphNode node = (GraphNode)it.next();
            node.write(buf);
        }

        buf.append("  </nodes>\n");
    }

    public void writeEdges(StringBuffer buf) {
        buf.append("  <edges>\n");

        Iterator it = edges.iterator();
        while (it.hasNext()) {
            GraphEdge edge = (GraphEdge)it.next();
            edge.write(buf);
        }

        buf.append("  </edges>\n");
    }
}
