/*
 *     GetDD.java  Apr 2, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.pdg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.Iterator;
 
public class GetDD {
    private PDG pdg;
    private CFG cfg;
    private GraphCompSet track = new GraphCompSet();   // CFGNode
    private GraphCompSet atrack = new GraphCompSet();  // CFGNode

    private GetDD() {
    }

    public GetDD(PDG pdg, CFG cfg) {
        this.pdg = pdg;
        this.cfg = cfg;
    }

    public void create() {
        Iterator it = cfg.getNodes().iterator();
        while(it.hasNext()) {
            CFGNode node = (CFGNode)it.next();
            if (node.hasDefVariable()) {
                findDDs(node);
            }
        }

        pdg.setSrcDstNodes();
    }
    
    private void findDDs(CFGNode node) {
        CFGStatement anchor = (CFGStatement)node;
        Iterator it = anchor.getDefVars().iterator();
        while(it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            findDDs(anchor, jv);
        }
    }

    private void findDDs(CFGNode anchor, JavaVariable jv) {

	// Thanks to Katsuhiko Yoshikawa
        Iterator it = anchor.getOutgoingEdges().iterator();
        while(it.hasNext()) {
            Flow flow = (Flow)it.next();
            if (!flow.isFallThrough()) {
                track.clear();
                CFGNode node = (CFGNode)flow.getDstNode();
                checkDD(anchor, node, jv);
            }
        }
    }

    private void checkDD(CFGNode anchor, CFGNode node, JavaVariable jv) {
        track.add(node);

        if (node.hasUseVariable()) {
            CFGStatement candidate = (CFGStatement)node;
            if (candidate.containsUseVariable(jv)) {
                DD edge = new DD(pdg.getNode(anchor), pdg.getNode(candidate), jv);
                PDGNode lc = getLoopCarried(anchor, candidate);

                if (anchor.isFormalInSt()) {
                    edge.setPI();
                } else if (candidate.isFormalOutSt()) {
                    edge.setPO();
                } else {
                    edge.setDU();
                }
                pdg.add(edge);

                if (lc != null && track.contains(lc.getCFGNode())) {
                    edge.setLoopCarriedNode(lc);
                }
            }
        }
        
        if (node.hasDefVariable()) {
            CFGStatement candidate = (CFGStatement)node;
            if (candidate.containsDefVariable(jv)) {
                return;
            }
        }

        Iterator it = node.getOutgoingEdges().iterator();
        while(it.hasNext()) {
            Flow flow = (Flow)it.next();
            if (!flow.isFallThrough()){
                CFGNode succ = (CFGNode)flow.getDstNode();
                if (!track.contains(succ))
                    checkDD(anchor, succ, jv);
            }
        }
    }

    private PDGNode getLoopCarried(CFGNode defNode, CFGNode useNode) {
        GraphCompSet defTrack = new GraphCompSet();
        atrack.clear();

        findCDAncestors(pdg.getNode(defNode), defTrack);
        if (defTrack.isEmpty()) {
            return null;
        }

        GraphCompSet useTrack = new GraphCompSet();
        atrack.clear();

        findCDAncestors(pdg.getNode(useNode), useTrack);
        if (useTrack.isEmpty()) {
            return null;
        }

        GetConstrainedReachableNodes reachablePath
          = new GetConstrainedReachableNodes(cfg, defNode, useNode);
        GraphCompSet commonNodes = new GraphCompSet();
        Iterator it = reachablePath.iterator();
        while (it.hasNext()) {
            CFGNode node = (CFGNode)it.next();
            PDGNode pnode = pdg.getNode(node);
            if (pnode != null && defTrack.contains(pnode) && useTrack.contains(pnode)) {
                commonNodes.add(pnode);
            }
        }
        if (commonNodes.isEmpty()) {
            return null;
        }

        PDGNode loopCarriedNode = (PDGNode)commonNodes.getFirst();  // nearest one
        return loopCarriedNode;
    }

    private void findCDAncestors(PDGNode anchor, GraphCompSet ancestors) {
        atrack.add(anchor);
        if (anchor.isLoop()) {
            ancestors.add(anchor);
        }

        Iterator it = anchor.getIncomingEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();
            if (edge.isCD()) {
                PDGNode node = (PDGNode)edge.getSrcNode();
                if (!atrack.contains(node)) {
                    findCDAncestors(node, ancestors);
                }
            }
        }
    }
}
