/*
 *     PDGNode.java  Apr 5, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.graph.pdg;
import org.jtool.platform.xml.graph.*;
import org.jtool.platform.xml.graph.cfg.*;
import org.jtool.platform.xml.wrapper.*;
import java.util.*;

public class PDGNode extends GraphNode {
    protected CFGNode cfgNode;

    protected PDGNode() {
    }

    public PDGNode(CFGNode node) {
        super();
        cfgNode = node;
        sort = cfgNode.getSort();
        id = cfgNode.getId();
    }

    public CFGNode getCFGNode() {
        return cfgNode;
    }

    public String getJxId() {
        return cfgNode.getJxId();
    }

    public boolean isLoop() {
        return cfgNode.isLoop();
    }

    public boolean isBranch() {
        return cfgNode.isBranch();
    }

    public boolean isStatement() {
        return false;
    }

    public GraphCompSet getIncomingCDEdges() {
        GraphCompSet edges = new GraphCompSet();

        Iterator it = getIncomingEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();

            if (edge.isCD()) {
                edges.add(edge);
            }
        }                
        return edges;
    }

    public GraphCompSet getOutgoingCDEdges() {
        GraphCompSet edges = new GraphCompSet();

        Iterator it = getOutgoingEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();

            if (edge.isCD()) {
                edges.add(edge);
            }
        }                
        return edges;
    }

    public GraphCompSet getIncomingDDEdges() {
        GraphCompSet edges = new GraphCompSet();

        Iterator it = getIncomingEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();

            if (edge.isDD()) {
                edges.add(edge);
            }
        }                
        return edges;
    }

    public GraphCompSet getOutgoingDDEdges() {
        GraphCompSet edges = new GraphCompSet();

        Iterator it = getOutgoingEdges().iterator();
        while (it.hasNext()) {
            Dependence edge = (Dependence)it.next();

            if (edge.isDD()) {
                edges.add(edge);
            }
        }                
        return edges;
    }

    public void print(String mesg, StringBuffer buf) {
        cfgNode.print(mesg, buf);
    }

    public void write(StringBuffer buf) {
        cfgNode.write(buf);
    }
}
