/*
 *     JavaStaticInit.java  Jun 19, 2004
 *
 *     Katsuhisa Maruyama (maru@jtool.org)
 */

package org.jtool.platform.xml.wrapper;
import org.jtool.platform.xml.JXConstants;
import org.jtool.platform.j2xml.Repository;
import org.w3c.dom.*;
import java.util.*;

public class JavaStaticInit extends JavaElement {
    private ArrayList locals = null;  // Node
    private ArrayList mcalls = null;  // Node
    private ArrayList vaccesses = null;  // Node
    private JavaClass jclass;

    protected JavaStaticInit() {
    }        

    public JavaStaticInit(Node node) {
        super(node);
        jclass = getJavaClass(elem);
    }

    public String getElemId() {
        return getElemId(elem);
    }

    public JavaClass getJavaClass() {
        return jclass;
    }

    public ArrayList getLocalNodes() {
        if (locals == null) {
            locals = getDescendantNodes(elem, JXConstants.LocalElem);
        }
        return locals;
    }

    public ArrayList getMethodCallNodes() {
        if (mcalls == null) {
            mcalls = getDescendantNodes(elem,
                JXConstants.ExpressionElem, JXConstants.MethodCall);
        }
        return mcalls;
    }

    public ArrayList getVariableAccessNodes() {
        if (vaccesses == null) {
            vaccesses = getDescendantNodes(elem,
                JXConstants.ExpressionElem, JXConstants.VarRef);
        }
        return vaccesses;
    }

    public JavaVariableList getVariableList() {
        JavaVariableList vars = new JavaVariableList();

        vars.addAll(getReferredVariables());
        vars.addAll(getLocalVariables());
        return vars;
    }

    private JavaVariableList getReferredVariables() {
        ArrayList nodes = getDescendantNodes(elem,
            JXConstants.ExpressionElem, JXConstants.VarRef);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    private JavaVariableList getLocalVariables() {
        ArrayList nodes = getDescendantNodes(elem, JXConstants.LocalElem);
        JavaVariableList vars = new JavaVariableList();

        Iterator it = nodes.iterator();
        while (it.hasNext()) {
            Node node = (Node)it.next();

            JavaVariable jv = new JavaVariable(node);
            vars.add(jv);
        }
        return vars;
    }

    public JavaVariableList getDefVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isDefVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaVariableList getUseVariableList() {
        JavaVariableList vars = new JavaVariableList();

        Iterator it = getVariableList().iterator();
        while (it.hasNext()) {
            JavaVariable jv = (JavaVariable)it.next();
            if (jv.isUseVariable()) {
                vars.add(jv);
            }
        }
        return vars;
    }

    public JavaClassList getUsedTypes() {
        JavaClassList classes = new JavaClassList();

        Iterator it = getTypeNodes().iterator();
        while (it.hasNext()) {
            Element elem = (Element)it.next();

            JavaType jt = new JavaType(elem);
            JavaClass jc = jt.getReferredJavaClass();
            if (jc != null) {
                classes.add(jc);
            }
        }
        return classes;
    }

    public boolean equals(JavaStaticInit js) {
        return js != null && isInTheSameClass(js); 
    }
}
